#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json
import pecan
from pecan import hooks

from mistral.db.v2 import api as db_api
from mistral.services import maintenance as maintenance_service

ALLOWED_WITHOUT_AUTH = ['/', '/v2/', '/health', '/maintenance']


def _complete_async_action(maintenance_mode, state):
    return (state.request.method == 'PUT' and
            '/v2/action_executions' in state.request.path and
            maintenance_mode == maintenance_service.PAUSING
            )


def _put_method_executions(maintenance_mode, state):
    if not (state.request.method == 'PUT' and
            '/v2/executions' in state.request.path):
        return False

    body = json.loads(state.request.body.decode('utf8').replace("'", '"'))
    return ('state' in body and body['state'] == 'CANCELLED' and
            (maintenance_mode == maintenance_service.PAUSING or
             maintenance_mode == maintenance_service.PAUSED
             ))


class MaintenanceHook(hooks.PecanHook):

    def before(self, state):
        if state.request.path in ALLOWED_WITHOUT_AUTH or \
                state.request.method == 'GET':
            return

        cluster_state = db_api.get_maintenance_status()

        if (cluster_state == maintenance_service.RUNNING or
                _complete_async_action(cluster_state, state) or
                _put_method_executions(cluster_state, state)):
            return

        msg = "Current Mistral state is {}. Method is not allowed".format(
            cluster_state
        )

        pecan.abort(
            status_code=423,
            detail=msg,
            headers={'Server-Error-Message': msg}
        )



# Copyright 2023 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""recreate mistral metrics table

Revision ID: 044
Revises: 043
Create Date: 2024-03-22 12:02:23.374515

"""

# revision identifiers, used by Alembic.
revision = '044'
down_revision = '043'

from alembic import op
import sqlalchemy as sa

from mistral.services import maintenance
from mistral_lib import utils


def upgrade():

    op.drop_table('mistral_metrics')

    mistral_metrics_table = op.create_table(
        'mistral_metrics',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('name', sa.String(length=255), nullable=False),
        sa.Column('value', sa.String(length=255), nullable=False),
        sa.UniqueConstraint('name'),
    )

    op.bulk_insert(mistral_metrics_table,
                   [
                       {
                           'created_at': utils.utc_now_sec(),
                           'updated_at': utils.utc_now_sec(),
                           'id': 1,
                           'name': 'maintenance_status',
                           'value': maintenance.RUNNING
                       }
                   ]
                   )
